import 'package:animate_do/animate_do.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:nuntio_hera/hera/models/find_options.dart';
import 'package:nuntio_hera/hera_ui/components/nuntio_fadein.dart';
import 'package:nuntio_hera/hera_ui/functions.dart';
import '../../../../hera.pb.dart';
import '../../../../hera.pbenum.dart';
import '../../../hera/hera.dart';
import '../../components/nuntio_button.dart';
import '../../components/nuntio_indicator.dart';
import '../../components/nuntio_text_field.dart';
import '../../models.dart';
import '../RegisterPage/register_page.dart';
import '../VerifyCodePage/verify_code_page.dart';
import 'components/continue_with_nuntio.dart';

class LoginPage extends StatefulWidget {
  LoginPage({
    Key? key,
    required this.identifierInputType,
    required this.nuntioFooter,
    required this.nuntioStyle,
    required this.nuntioText,
    required this.nuntioColor,
    required this.nuntioTextStyle,
    required this.logo,
    required this.config,
    required this.background,
    required this.onLogin,
    required this.onRegister,
    required this.supportedAuthMechanisms,
    this.padding,
    this.brightness,
  }) {}

  // style and text config
  final HeraStyle nuntioStyle;
  final HeraText nuntioText;
  final HeraColor nuntioColor;
  final HeraTextStyle nuntioTextStyle;
  final HeraFooter nuntioFooter;
  final List<LoginType> supportedAuthMechanisms;

  // general
  final Widget logo;
  final Config config;
  final TextInputType identifierInputType;
  final Brightness? brightness;
  final EdgeInsetsGeometry? padding;

  // style
  final BoxDecoration background;

  // functions.dart
  final Function(BuildContext context) onLogin;
  final Function(BuildContext context) onRegister;

  @override
  State<LoginPage> createState() => _LoginPageState();
}

class _LoginPageState extends State<LoginPage> {
  final _identifierController = TextEditingController();
  final passwordController = TextEditingController();
  final verifyCodeController = TextEditingController();
  String disabledTipnote = "";
  bool isLoading = false;

  loginSuccess() {
    setState(() {
      isLoading = false;
    });
    passwordController.text = "";
    _identifierController.text = "";
    widget.onLogin(context);
  }

  loginFailure() {
    setState(() {
      isLoading = false;
    });
    passwordController.text = "";
    _identifierController.text = "";
  }

  void getLoginDisabledTipnote() {
    if (passwordController.text == "") {
      setState(() {
        disabledTipnote = widget.nuntioText.invalidPasswordText;
      });
    } else if (_identifierController.text == "") {
      setState(() {
        disabledTipnote = widget.nuntioText.invalidIdentifierText;
      });
    }
    return;
  }

  @override
  void initState() {
    super.initState();
    getLoginDisabledTipnote();
  }

  @override
  Widget build(BuildContext context) {
    var size = MediaQuery.of(context).size;
    return CupertinoPageScaffold(
      child: Container(
        decoration: widget.background,
        child: SingleChildScrollView(
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.center,
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              SizedBox(),
              Center(
                child: Container(
                  constraints: BoxConstraints(
                      maxWidth: 450,
                      minHeight: size.height - widget.nuntioFooter.height),
                  margin: const EdgeInsets.only(
                      left: 25, right: 25, bottom: 20, top: 0),
                  padding: widget.padding,
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.center,
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      NuntioFadeIn(child: widget.logo),
                      if (widget.nuntioText.loginTitle != "")
                        NuntioFadeIn(
                          delay: 5,
                          padding: const EdgeInsets.only(bottom: 10),
                          child: Text(
                            widget.nuntioText.loginTitle,
                            style: widget.nuntioTextStyle.titleStyle,
                            textAlign: TextAlign.center,
                          ),
                        ),
                      if (widget.nuntioText.loginSubtitle != "")
                        NuntioFadeIn(
                          delay: 6,
                          padding: const EdgeInsets.only(bottom: 15),
                          child: Text(
                            widget.nuntioText.loginSubtitle,
                            style: widget.nuntioTextStyle.subtitleStyle,
                            textAlign: TextAlign.center,
                          ),
                        ),
                      if (widget.config.oidcProviders.isNotEmpty)
                        NuntioFadeIn(
                          delay: 6,
                          child: Column(
                            children: [
                              Padding(
                                padding: EdgeInsets.only(bottom: 10),
                                child: Align(
                                  alignment: Alignment.centerLeft,
                                  child: Text(
                                    widget.nuntioText.continueWithText,
                                    style: widget.nuntioTextStyle.labelStyle,
                                    textAlign: TextAlign.left,
                                  ),
                                ),
                              ),
                              Column(
                                children: widget.config.oidcProviders
                                    .map((e) => Padding(
                                          padding:
                                              const EdgeInsets.only(bottom: 10),
                                          child: ContinueWithButton(
                                              heraStyle: widget.nuntioStyle,
                                              heraColor: widget.nuntioColor,
                                              heraTextStyle:
                                                  widget.nuntioTextStyle,
                                              oidcProvider: e,
                                              loginSuccess: loginSuccess),
                                        ))
                                    .toList(),
                              )
                            ],
                          ),
                        ),
                      if (!widget.config.disableLogin &&
                          containsIdPassword(
                              supportedAuthMechanisms:
                                  widget.supportedAuthMechanisms))
                        NuntioFadeIn(
                          delay: 7,
                          child: SizedBox(
                            height: 75,
                            width: widget.nuntioStyle.buttonWidth,
                            child: Row(
                              crossAxisAlignment: CrossAxisAlignment.center,
                              mainAxisAlignment: MainAxisAlignment.center,
                              children: [
                                Expanded(
                                  child: Divider(
                                    color: widget.nuntioTextStyle
                                        .oidcButtonTextStyle.color
                                        ?.withOpacity(0.2),
                                  ),
                                ),
                                SizedBox(width: 15),
                                Text(
                                  widget.nuntioText.orLoginText,
                                  style: widget.nuntioTextStyle.labelStyle
                                      .copyWith(
                                          fontWeight: FontWeight.normal,
                                          color: widget.nuntioTextStyle
                                              .oidcButtonTextStyle.color
                                              ?.withOpacity(0.6)),
                                  textAlign: TextAlign.left,
                                ),
                                SizedBox(width: 15),
                                Expanded(
                                  child: Divider(
                                    color: widget.nuntioTextStyle
                                        .oidcButtonTextStyle.color
                                        ?.withOpacity(0.2),
                                  ),
                                ),
                              ],
                            ),
                          ),
                        ),
                      if (!widget.config.disableLogin &&
                          containsIdPassword(
                              supportedAuthMechanisms:
                                  widget.supportedAuthMechanisms))
                        NuntioFadeIn(
                          delay: 8,
                          child: AutofillGroup(
                            child: Column(
                              children: [
                                NuntioTextField(
                                  scrollPadding: EdgeInsets.only(bottom: 210),
                                  brightness: widget.brightness,
                                  nuntioTextStyle: widget.nuntioTextStyle,
                                  nuntioStyle: widget.nuntioStyle,
                                  nuntioColor: widget.nuntioColor,
                                  controller: _identifierController,
                                  hint: widget.nuntioText.identifierLabelText,
                                  textInputType: widget.identifierInputType,
                                  label: widget.nuntioText.identifierLabelText,
                                  prefix: FontAwesomeIcons.fingerprint,
                                  autofillHints: [
                                    AutofillHints.username,
                                    AutofillHints.email,
                                    AutofillHints.telephoneNumber
                                  ],
                                  textInputAction: TextInputAction.next,
                                ),
                                NuntioTextField(
                                  scrollPadding: EdgeInsets.only(bottom: 100),
                                  brightness: widget.brightness,
                                  nuntioTextStyle: widget.nuntioTextStyle,
                                  padding: EdgeInsets.zero,
                                  nuntioStyle: widget.nuntioStyle,
                                  nuntioColor: widget.nuntioColor,
                                  controller: passwordController,
                                  hint: widget.nuntioText.passwordLabelText,
                                  textInputType: TextInputType.text,
                                  label: widget.nuntioText.passwordLabelText,
                                  autofillHints: [AutofillHints.password],
                                  textInputAction: TextInputAction.done,
                                  prefix: FontAwesomeIcons.key,
                                  obscureText: true,
                                  onSubmitted: (_) {
                                    onLogin();
                                  },
                                ),
                                Align(
                                  alignment: Alignment.centerRight,
                                  child: Padding(
                                    padding: const EdgeInsets.only(bottom: 10),
                                    child: NuntioButton(
                                      padding: EdgeInsets.all(0),
                                      child: Text(
                                        widget.nuntioText
                                            .forgotPasswordButtonText,
                                        textAlign: TextAlign.right,
                                      ),
                                      onPressed: () {},
                                      color:
                                          widget.brightness == Brightness.light
                                              ? CupertinoColors.black
                                              : CupertinoColors.white,
                                    ),
                                  ),
                                ),
                              ],
                            ),
                          ),
                        ),
                      if (!widget.config.disableLogin &&
                          containsIdPassword(
                              supportedAuthMechanisms:
                                  widget.supportedAuthMechanisms))
                        NuntioFadeIn(
                          delay: 8,
                          child: NuntioButton(
                            width: widget.nuntioStyle.buttonWidth,
                            height: widget.nuntioStyle.buttonHeight,
                            color: widget.nuntioColor.primaryColor,
                            filled: true,
                            onPressed: onLogin,
                            child: isLoading
                                ? NuntioIndicator(
                                    color: widget.nuntioTextStyle
                                        .loginButtonTextStyle.color)
                                : Row(
                                    crossAxisAlignment:
                                        CrossAxisAlignment.center,
                                    mainAxisAlignment: MainAxisAlignment.center,
                                    children: [
                                      Text(
                                        widget.nuntioText.loginButtonText,
                                        style: widget.nuntioTextStyle
                                            .loginButtonTextStyle,
                                        textAlign: TextAlign.center,
                                      ),
                                      SizedBox(width: 8),
                                      FaIcon(
                                        FontAwesomeIcons.arrowRight,
                                        size: 15,
                                        color: widget.nuntioTextStyle
                                            .loginButtonTextStyle.color,
                                      ),
                                    ],
                                  ),
                          ),
                        ),
                      if (!widget.config.disableSignup &&
                          containsIdPassword(
                              supportedAuthMechanisms:
                                  widget.supportedAuthMechanisms))
                        NuntioFadeIn(
                          delay: 9,
                          child: NuntioButton(
                            width: widget.nuntioStyle.buttonWidth,
                            height: widget.nuntioStyle.buttonHeight,
                            color: widget.nuntioColor.primaryColor,
                            disabled: isLoading,
                            onPressed: () {
                              Navigator.push(
                                context,
                                CupertinoPageRoute(
                                  builder: (context) => RegisterPage(
                                    nuntioText: widget.nuntioText,
                                    logo: widget.logo,
                                    supportedAuthMechanisms:
                                        widget.supportedAuthMechanisms,
                                    nuntioStyle: widget.nuntioStyle,
                                    nuntioColor: widget.nuntioColor,
                                    background: widget.background,
                                    //onRegister: widget.onRegister,
                                    config: widget.config,
                                    brightness: widget.brightness,
                                    nuntioTextStyle: widget.nuntioTextStyle,
                                    nuntioFooter: widget.nuntioFooter,
                                    onLogin: widget.onLogin,
                                    onRegister: widget.onRegister,
                                    //onLogin: widget.onLogin,
                                  ),
                                ),
                              );
                            },
                            child: Text(
                              widget.nuntioText.dontHaveAnAccountButtonText,
                              textAlign: TextAlign.center,
                            ),
                          ),
                        ),
                    ],
                  ),
                ),
              ),
              SizedBox(
                height: widget.nuntioFooter.height,
                child: widget.nuntioFooter.footer,
              ),
            ],
          ),
        ),
      ),
    );
  }

  onLogin() {
    if (isLoading) {
      return;
    }
    setState(() {
      isLoading = true;
    });
    FindOptions _findOptions = FindOptions();
    if (widget.supportedAuthMechanisms.contains(LoginType.EMAIL_PASSWORD) &&
        validateEmail(_identifierController.text)) {
      _findOptions.email = _identifierController.text;
    } else if (widget.supportedAuthMechanisms
            .contains(LoginType.EMAIL_PASSWORD) &&
        validateMobile(_identifierController.text)) {
      _findOptions.phone = _identifierController.text;
    } else if (widget.supportedAuthMechanisms
        .contains(LoginType.USERNAME_PASSWORD)) {
      _findOptions.username = _identifierController.text;
    }
    Hera.login(password: passwordController.text, findOptions: _findOptions)
        .catchError((err) {
      loginFailure();
      showCupertinoDialog(
          context: context,
          builder: (context) {
            return CupertinoAlertDialog(
              title: Text(
                widget.nuntioText.errorTitle,
              ),
              content: Text(
                widget.nuntioText.errorSubtitle,
              ),
              actions: <Widget>[
                CupertinoButton(
                    onPressed: () {
                      Navigator.of(context).pop();
                    },
                    child: const Text(
                      'okay',
                      style: TextStyle(color: black),
                    ))
              ],
            );
          });
    }).then((loginSession) {
      if (loginSession.loginStatus == LoginStatus.EMAIL_IS_NOT_VERIFIED) {
        Navigator.of(context)
            .push(
          CupertinoPageRoute(
            builder: (context) => VerifyCodePage(
              loginType: LoginType.EMAIL_PASSWORD,
              identifier: _identifierController.text,
              emailExpiresAt: loginSession.emailExpiresAt.toDateTime().toUtc(),
              config: widget.config,
              onLogin: widget.onLogin,
              nuntioStyle: widget.nuntioStyle,
              nuntioColor: widget.nuntioColor,
              background: widget.background,
              nuntioText: widget.nuntioText,
              nuntioFooter: widget.nuntioFooter,
              nuntioTextStyle: widget.nuntioTextStyle,
              password: passwordController.text,
              logo: widget.logo,
            ),
          ),
        )
            .then((_) {
          setState(() {
            isLoading = false;
          });
          passwordController.text = "";
          _identifierController.text = "";
        });
      } else {
        loginSuccess();
      }
    });
  }
}
